﻿using System;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.Text;

public class WebServiceSoapClient<T> : IDisposable
{
    private readonly T myChannel;
    private readonly IClientChannel myClientChannel;

    public WebServiceSoapClient(string url)
        : this(url, null)
    {
    }

    public WebServiceSoapClient(string url,
         Action<CustomBinding, HttpTransportBindingElement, EndpointAddress, ChannelFactory> init)
    {

        //transport http o https
        var transport = url.StartsWith("http", StringComparison.InvariantCultureIgnoreCase)
                            ? new HttpTransportBindingElement()
                            : new HttpsTransportBindingElement();

        
        var binding = new CustomBinding();
        //binding soap
        binding.Elements.Add(new TextMessageEncodingBindingElement(MessageVersion.Soap12, Encoding.UTF8));
        binding.Elements.Add(transport);
        var address = new EndpointAddress(url);
       
        //canale verso il webservice
        var factory = new ChannelFactory<T>(binding, address);
        if (init != null)
        {
            init(binding, transport, address, factory);
        }

        this.myClientChannel = (IClientChannel)factory.CreateChannel();
        this.myChannel = (T)this.myClientChannel;


    }

    public void Dispose()
    {
        this.myClientChannel.Dispose();
    }

    public IClientChannel ClientChannel
    {
        get { return this.myClientChannel; }
    }

    public T Channel
    {
        get { return this.myChannel; }
    }



}
